//+------------------------------------------------------------------+
//|                                             moving average tests |
//+------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1  PaleVioletRed
#property indicator_width1  2
#property indicator_minimum 0

extern int Periods = 14;
extern int Price   = PRICE_CLOSE;
extern int Method  = MODE_SMA;

double buffer1[];
double buffer2[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorDigits(5); 
   IndicatorBuffers(2);
      SetIndexBuffer(0,buffer1);
      SetIndexBuffer(1,buffer2);
   IndicatorShortName(getAverageName(Method)+" ("+Periods+")");
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      buffer2[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      buffer1[i] = iCMAOnArray(buffer2,buffer1,Periods,Method,i);
   }      
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iCMAOnArray(double& source[], double& result[], int period, int method, int i)
{
   switch(method)
   {
      case 0: return(iSmaOnArray(source,period,i));
      case 1: return(iEmaOnArray(source,result,period,i));
      case 2: return(iSmmaOnArray(source,result,period,i));
      case 3: return(iLwmaOnArray(source,period,i));
      case 4: return(iTmaOnArray(source,period,i));
      case 5: return(iSineWMAOnArray(source,period,i));
      case 6: return(iVolumeWMAOnArray(source,period,i));
      case 7: return(iEPMAOnArray(source,period,i));
   }
   return(EMPTY_VALUE);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iSmaOnArray(double& array[], double period, int i)
{
   double sum = 0; 
   for(int k=0; k<period && (i+k)<Bars; k++) sum += array[i+k];  
   if (k!=0)
         return(sum/k);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iEmaOnArray(double& source[], double& result[], double period, int i)
{
   double alpha = 2.0 / (1.0+period);
          result[i] = result[i+1]+alpha*(source[i]-result[i+1]);
   return(result[i]);
}
//
//
//
//
//

double iSmmaOnArray(double& source[], double& result[], double period, int i)
{
   if (i>=(Bars-period))
   {
      double sum = 0; 
         for(int k=0; k<period && (i+k)<Bars; k++) sum += source[i+k];  
         if (k!=0)
               result[i] = sum/k;
         else  result[i] = EMPTY_VALUE;
   }      
   else   result[i] = (result[i+1]*(period-1)+source[i])/period;
   return(result[i]);
}

//
//
//
//
//

double iLwmaOnArray(double& array[], double period, int i)
{
   double sum  = 0;
   double sumw = 0;

   for(int k=0; k<period && (i+k)<Bars; k++)
   {
      double weight = period-k;
             sumw  += weight;
             sum   += weight*array[i+k];  
   }             
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iTmaOnArray(double& array[], double period, int i)
{
   double half = (period+1.0)/2.0;
   double sum  = 0;
   double sumw = 0;

   for(int k=0; k<period && (i+k)<Bars; k++)
   {
      double weight = k+1; if (weight > half) weight = period-k;
             sumw  += weight;
             sum   += weight*array[i+k];  
   }             
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

#define Pi 3.14159265358979323846
double iSineWMAOnArray(double& array[], int period, int i)
{
   double sum  = 0;
   double sumw = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
      double weight = MathSin(Pi*(k+1)/(period+1));
             sumw  += weight;
             sum   += weight*array[i+k]; 
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iVolumeWMAOnArray(double& array[], int period, int i)
{
   double sum  = 0;
   double sumw = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
      double weight = Volume[i+k];
             sumw  += weight;
             sum   += weight*array[i+k]; 
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iEPMAOnArray(double& array[], int period, int i)
{
   double weight = 2.0*period-1.0;
   double sum    = 0;
   double sumw   = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
       sumw   += weight;
       sum    += weight*array[i+k]; 
       weight -= 3;
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string methodNames[] = {"SMA","EMA","SMMA","LWMA","TriMA","SWMA","VWMA","EPMA"};
string getAverageName(int& method)
{
   method=MathMax(MathMin(method,7),0); return(methodNames[method]);
}